@@ -20,6 +20,7 @@ from scipy.constants import speed_of_light
 
 from sarpy.geometry.geocoords import geodetic_to_ecf
 
+import sarpy._extensions
 from sarpy.io.complex.base import SICDTypeReader
 from sarpy.io.complex.other_nitf import ComplexNITFReader
 from sarpy.io.complex.sicd_elements.blocks import Poly1DType, Poly2DType
@@ -70,6 +71,22 @@ def _format_class_str(class_str: str) -> str:
         return class_str
 
 
+def load_addin():
+    """Check for a Radarsat addin module"""
+    try:
+        from sarpy.io.complex import radarsat_addin
+        return radarsat_addin
+    except ImportError:
+        pass
+
+    eps = sarpy._extensions.entry_points(group='sarpy.io.complex.radarsat')
+    if not eps:
+        return None
+    if len(eps) > 1:
+        raise RuntimeError("More than one Radarsat addin found")
+    return list(eps)[0].load()
+
+
 def _validate_segment_and_sicd(
         the_sicd: SICDType,
         data_segment: DataSegment,
@@ -535,10 +552,7 @@ class RadarSatDetails(object):
         collection_info : CollectionInfoType
         """
 
-        try:
-            import sarpy.io.complex.radarsat_addin as radarsat_addin
-        except ImportError:
-            radarsat_addin = None
+        radarsat_addin = load_addin()
 
         collector_name = self.satellite
         start_time_dt = start_time.astype(datetime)
