@@ -5,17 +5,16 @@ This module contains the base objects for use in base xml/serializable functiona
 __classification__ = "UNCLASSIFIED"
 __author__ = "Thomas McCullough"
 
-import logging
-from xml.etree import ElementTree
-import json
-from datetime import date, datetime
-from collections import OrderedDict
 import copy
-import re
-from io import StringIO
-from typing import Dict, Optional
-
+import json
+import logging
 import numpy
+import re
+from collections import OrderedDict
+from datetime    import date, datetime
+from io          import StringIO
+from typing      import Dict, Optional
+from xml.etree   import ElementTree
 
 from sarpy.compliance import bytes_to_string
 
@@ -37,27 +36,21 @@ DEFAULT_STRICT = False
 
 def get_node_value(nod: ElementTree.Element) -> Optional[str]:
     """
-    XML parsing helper for extracting text value from an ElementTree Element. No error checking performed.
-
+    Extracts and returns the stripped text value from an ElementTree Element.
+    Returns None if the text is None or only whitespace.
     Parameters
     ----------
     nod : ElementTree.Element
-        the xml dom element
-
+        The XML DOM element.
     Returns
     -------
-    str
-        the string value of the node.
+    Optional[str]
+        The stripped string value of the node, or None.
     """
-
-    if nod.text is None:
-        return None
-
-    val = nod.text.strip()
-    if len(val) == 0:
-        return None
-    else:
-        return val
+    if nod.text:
+        val = nod.text.strip()
+        return val if val else None
+    return None
 
 
 def create_new_node(
@@ -123,10 +116,10 @@ def create_text_node(
 
 
 def find_first_child(
-        node: ElementTree.Element,
-        tag: str,
-        xml_ns: Optional[Dict[str, str]],
-        ns_key: Optional[str]) -> ElementTree.Element:
+        node  : ElementTree.Element,
+        tag   : str,
+        xml_ns: Optional[Dict[str, str]] = None,
+        ns_key: Optional[str]            = None) -> ElementTree.Element:
     """
     Finds the first child node
 
@@ -134,8 +127,8 @@ def find_first_child(
     ----------
     node : ElementTree.Element
     tag : str
-    xml_ns : None|dict
-    ns_key : None|str
+    xml_ns : None|dict XML namespace of the node in question
+    ns_key : None|str  Namespace key to use to preface the tag
 
     Returns
     -------
@@ -150,7 +143,11 @@ def find_first_child(
         return node.find('{}:{}'.format(ns_key, tag), xml_ns)
 
 
-def find_children(node, tag, xml_ns, ns_key):
+def find_children(
+        node  : ElementTree.Element,
+        tag   : str,
+        xml_ns: Optional[Dict[str, str]] = None,
+        ns_key: Optional[str]            = None):
     """
     Finds the collection of children nodes
 
@@ -290,6 +287,40 @@ def validate_xml_from_file(xml_path, xsd_path, output_logger=None):
 
 
 def parse_str(value, name, instance):
+    """
+    The parse_str function is not a generic string parser. It is a helper 
+    function specifically for parsing values within XML elements in 
+    sarpy.io.xml.base. It's used internally by the library to extract text 
+    values from XML ElementTree objects. 
+    The function is not intended for public use for general string parsing, and 
+    attempting to use it on a simple text string will just return the initial 
+    string as parse_str expects an XML element as its input. 
+
+    Parameters
+    ----------
+    value : ElementTree.Element|None|str
+        The ElementTree.Element entity that you want to get the value from.
+        None returns None.
+        A string value will return the given string unchanged.
+    name : str 
+        Name of the field to return the value of. This is only used in the 
+        raised error message
+    instance :
+        The class of the variable. This is only used in the raised error message.
+
+    Returns
+    -------
+    None | str
+        Returns None if value passed is None. Returns the string passed if value
+        is a string. Returns the string value of a node when passed an 
+        ElementTree.Element
+
+    Raises
+    -------
+    TypeError
+        When passed a value with a type other than the expected input types.
+    """
+
     if value is None:
         return None
     if isinstance(value, str):
@@ -303,6 +334,35 @@ def parse_str(value, name, instance):
 
 
 def parse_bool(value, name, instance):
+    """
+    The parse_bool function is a helper function specifically for parsing boolean
+    values within XML elements in sarpy.io.xml.base. 
+    The function is not intended for public use.
+    
+    Parameters
+    ----------
+    value : ElementTree.Element|None|str[0, 1, true, false]
+        The ElementTree.Element entity that you want to get the value from.
+        None returns None.
+        A string value will return the boolean value for the string if it can be 
+        converted to a boolean.
+    name : str 
+        Name of the field to return the value of. This is only used in the 
+        raised error message
+    instance :
+        The class of the variable. This is only used in the raised error message.
+
+    Returns
+    -------
+    None | bool
+        Returns None if value passed is None. Returns the boolean value
+        of a node when passed an ElementTree.Element
+
+    Raises
+    -------
+    ValueError
+        When passed a value with a type other than the expected input types.
+    """
     def parse_string(val):
         if val.lower() in ['0', 'false']:
             return False
@@ -330,6 +390,36 @@ def parse_bool(value, name, instance):
 
 
 def parse_int(value, name, instance):
+    """
+    The parse_int function is a helper function specifically for parsing integer
+    values within XML elements in sarpy.io.xml.base. 
+    The function is not intended for public use. This function is recursive.
+    
+    Parameters
+    ----------
+    value : ElementTree.Element|None|int|str
+        The ElementTree.Element entity that you want to get the value from.
+        None returns None.
+        int returns the integer.
+        A string value will return the integer value for the string if it can be 
+        converted to an integer.
+    name : str 
+        Name of the field to return the value of. This is only used in the 
+        raised error message
+    instance :
+        The class of the variable. This is only used in the raised error message.
+
+    Returns
+    -------
+    None | bool
+        Returns None if value passed is None. Returns the boolean value
+        of a node when passed an ElementTree.Element
+
+    Raises
+    -------
+    TypeError
+        When passed a value with a type other than the expected input types.
+    """
     if value is None:
         return None
     if isinstance(value, int):
@@ -357,6 +447,36 @@ def parse_int(value, name, instance):
 
 # noinspection PyUnusedLocal
 def parse_float(value, name, instance):
+    """
+    The parse_float function is a helper function specifically for parsing float
+    values within XML elements in sarpy.io.xml.base. 
+    The function is not intended for public use. This function is recursive.
+    
+    Parameters
+    ----------
+    value : ElementTree.Element|None|float|str
+        The ElementTree.Element entity that you want to get the value from.
+        None returns None.
+        Float returns a float.
+        A string value will return the float value for the string if it can be 
+        converted to a float.
+    name : str 
+        Name of the field to return the value of. This is only used in the 
+        raised error message
+    instance :
+        The class of the variable. This is only used in the raised error message.
+
+    Returns
+    -------
+    None | bool
+        Returns None if value passed is None. Returns the float value
+        of a node when passed an ElementTree.Element
+
+    Raises
+    -------
+    TypeError
+        When passed a value with a type other than the expected input types.
+    """
     if value is None:
         return None
     if isinstance(value, float):
@@ -370,6 +490,36 @@ def parse_float(value, name, instance):
 
 
 def parse_complex(value, name, instance):
+    """
+    The parse_complex function is a helper function specifically for parsing 
+    complex number (numbers with both a real and an imaginary component) values 
+    within XML elements in sarpy.io.xml.base. 
+    The function is not intended for public use. This function is recursive.
+    
+    Parameters
+    ----------
+    value : ElementTree.Element|None|complex|dict
+        The ElementTree.Element entity that you want to get the value from.
+        None returns None.
+        complex returns a complex.
+        dict is a dictionary representation of a complex number.
+    name : str 
+        Name of the field to return the value of. This is only used in the 
+        raised error message
+    instance :
+        The class of the variable. This is only used in the raised error message.
+
+    Returns
+    -------
+    None | bool
+        Returns None if value passed is None. Returns the complex value
+        of a node when passed an ElementTree.Element
+
+    Raises
+    -------
+    TypeError
+        When passed a value with a type other than the expected input types.
+    """
     if value is None:
         return None
     if isinstance(value, complex):
@@ -416,6 +566,36 @@ def parse_complex(value, name, instance):
 
 
 def parse_datetime(value, name, instance, units='us'):
+    """
+    The parse_datetime function is a helper function specifically for parsing 
+    datetime values within XML elements in sarpy.io.xml.base. 
+    The function is not intended for public use. This function is recursive.
+    
+    Parameters
+    ----------
+    value : ElementTree.Element|None|datetime|dict
+        The ElementTree.Element entity that you want to get the value from.
+        None returns None.
+        datetime returns a datetime.
+        A string value will return the datetime value for the string if it can be 
+        converted to a datetime.
+    name : str 
+        Name of the field to return the value of. This is only used in the 
+        raised error message
+    instance :
+        The class of the variable. This is only used in the raised error message.
+
+    Returns
+    -------
+    None | bool
+        Returns None if value passed is None. Returns the float value
+        of a node when passed an ElementTree.Element
+
+    Raises
+    -------
+    TypeError
+        When passed a value with a type other than the expected input types.
+    """
     if value is None:
         return None
     if isinstance(value, numpy.datetime64):
@@ -429,8 +609,10 @@ def parse_datetime(value, name, instance, units='us'):
     elif isinstance(value, ElementTree.Element):
         # from XML deserialization - extract the string
         return parse_datetime(get_node_value(value), name, instance, units=units)
-    elif isinstance(value, (date, datetime, numpy.int64, numpy.float64)):
+    elif isinstance(value, (date, datetime)):
         return numpy.datetime64(value, units)
+    elif isinstance(value, (numpy.int64, numpy.float64)):
+        return numpy.datetime64(int(value.item()), units)
     elif isinstance(value, int):
         # this is less safe, because the units are unknown...
         return numpy.datetime64(value, units)
@@ -533,7 +715,7 @@ def parse_serializable_array(value, name, instance, child_type, child_tag):
                 return numpy.array([child_type(Coefs=array) for array in value], dtype='object')
             else:
                 raise ValueError(
-                    'Attribute {} of array type functionality belonging to class {} got an list '
+                    'Attribute {} of array type functionality belonging to class {} got a list '
                     'containing elements type {} and construction failed.'.format(
                         name, instance.__class__.__name__, type(value[0])))
         else:
@@ -1055,7 +1237,7 @@ class Serializable(object):
                 # again, I have no idea how we'd find ourselves here, unless inconsistencies have been introduced
                 # into the descriptor
                 raise ValueError(
-                    'The value associated with attribute {} is an instance of class {}, if None, is required to be'
+                    'The value associated with attribute {} is an instance of class {}, if None, is required to be '
                     'a one-dimensional numpy.ndarray, but it has shape {}'.format(
                         attribute, self.__class__.__name__, val.shape))
             if val.size == 0:
@@ -1074,7 +1256,7 @@ class Serializable(object):
                 # I have no idea how we'd find ourselves here, unless inconsistencies have been introduced
                 # into the descriptor
                 raise ValueError(
-                    'The value associated with attribute {} is an instance of class {}, if None, is required to be'
+                    'The value associated with attribute {} is an instance of class {}, if None, is required to be '
                     'a numpy.ndarray of dtype float64 or object, but it has dtype {}'.format(
                         attribute, self.__class__.__name__, val.dtype))
 
@@ -1232,7 +1414,7 @@ class Serializable(object):
                 # again, I have no idea how we'd find ourselves here, unless inconsistencies have been introduced
                 # into the descriptor
                 raise ValueError(
-                    'The value associated with attribute {} is an instance of class {}, if None, is required to be'
+                    'The value associated with attribute {} is an instance of class {}, if None, is required to be '
                     'a one-dimensional numpy.ndarray, but it has shape {}'.format(
                         attribute, self.__class__.__name__, val.shape))
 
@@ -1245,7 +1427,7 @@ class Serializable(object):
                 # I have no idea how we'd find ourselves here, unless inconsistencies have been introduced
                 # into the descriptor
                 raise ValueError(
-                    'The value associated with attribute {} is an instance of class {}. This is expected to be'
+                    'The value associated with attribute {} is an instance of class {}. This is expected to be '
                     'a numpy.ndarray of dtype float64, but it has dtype {}'.format(
                         attribute, self.__class__.__name__, val.dtype))
 
@@ -1276,7 +1458,7 @@ class Serializable(object):
                 return val.isoformat(sep='T')
             else:
                 raise ValueError(
-                    'a entry for class {} using tag {} is of type {}, and serialization has not '
+                    'An entry for class {} using tag {} is of type {}, and serialization has not '
                     'been implemented'.format(self.__class__.__name__, field, type(val)))
 
         if check_validity:
